/*Class name: DVMObject
 *Namespace: DVM_Management_Center.Objects
 *Author: Harald Gsoedl
 *Last change: Benno Rott
 *Changed: 07/02/2007
 *Use: This class represents a DVM-Object and fills it with informations
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Windows.Forms;
using System.Threading;
using System.Xml.Serialization;

using DVM_Management_Center.SNMP;
using DVM_Management_Center.Icons;

namespace DVM_Management_Center.Objects
{
    public class DVMObject
    {
        /// <summary>
        /// The class represents a DVM-Object
        /// </summary>
        public DVMObject()
        {
            
        }

        [XmlIgnore()]        
        private String _siteName;
        [XmlElement("siteName", DataType = "string")]
        public String SiteName
        {
            get { return _siteName; }
            set { _siteName = value; }              
        }

        [XmlIgnore()]
        private String _ip;
        [XmlElement("ip", DataType = "string")]
        public String Ip
        {
            get { return _ip; }
            set { _ip = value; }
        }

        [XmlIgnore()]
        private String _rComm;
        [XmlElement("rComm", DataType = "string")]
        public String RComm
        {
            get { return _rComm; }
            set { _rComm = value; }
        }

        [XmlIgnore()]
        private String _wComm;
        [XmlElement("wComm", DataType = "string")]
        public String WComm
        {
            get { return _wComm; }
            set { _wComm = value; }
        }

        [XmlIgnore()]
        private String _top;
        [XmlElement("top", DataType = "string")]
        public String Top
        {
            get { return _top; }
            set { _top = value; }
        }

        [XmlIgnore()]
        private String _left;
        [XmlElement("left", DataType = "string")]
        public String Left
        {
            get { return _left; }
            set { _left = value; }
        }

        [XmlIgnore()]
        private System.Windows.Forms.Label _led;
        [XmlIgnore()]
        public System.Windows.Forms.Label Led
        {
            get { return _led; }
            set { _led = value; }
        }

        [XmlIgnore()]
        private System.Windows.Forms.Label _mapText;
        [XmlIgnore()]
        public System.Windows.Forms.Label MapText
        {
            get { return _mapText; }
            set { _mapText = value; }
        }

        [XmlIgnore()]
        private System.Windows.Forms.TreeNode _dvmNode;
        [XmlIgnore()]
        public System.Windows.Forms.TreeNode DvmNode
        {
            get { return _dvmNode; }
        }
        [XmlIgnore()]
        private bool _ready;
        [XmlIgnore()]
        public bool Ready
        {
            get { return _ready; }
            set { _ready = value; }
        }

        /// <summary>
        /// Initialize the TreeNode-object
        /// </summary>
        public void InitNode()
        {
            _dvmNode = new TreeNode();
            _dvmNode.Text = _siteName;
            _dvmNode.ImageIndex = IconImagelist.MAP_NO_MONITORING;
            _dvmNode.SelectedImageIndex = IconImagelist.MAP_NO_MONITORING;
            _dvmNode.Tag = this._dvmNode.Text + this._dvmNode.ImageIndex.ToString();
        }

        /// <summary>
        /// Save all infos about the DVMs in the TreeNode
        /// </summary>
        public void SetDVM()
        {
            DVMAppFuncs dvm = new DVMAppFuncs(_ip, _rComm, _wComm, Application.StartupPath);
            _ready = false;
            _dvmNode = new TreeNode();
            
            dvm.SetAllInputInfos();

            if (dvm.InpInfos != null)
            {
                int dvmNode_redCnt = 0;
                int dvmNode_yelCnt = 0;
                int dvmNode_greCnt = 0;
                int dvmNode_syncCnt = 0;
                int dvmNode_disCnt = 0;
                int dvmNode_nmCnt = 0;
                _dvmNode.Name = _ip;
                _dvmNode.ImageIndex = 7;
                _dvmNode.SelectedImageIndex = 7;

                TreeNode node = _dvmNode;
                foreach (Object o in dvm.InpInfos.Values)
                { // Analyzer + SiteName
                    if (o.GetType() == String.Empty.GetType())
                    { // Sitename
                        _dvmNode.Text = _siteName = ((String)o == String.Empty) ? Ip : (String)o;
                    }
                    else
                    { // Analyzer
                        Hashtable hash = new Hashtable();
                        hash = (Hashtable)o;
                        for (int i = 1; i <= hash.Count; i++)
                        { // Sort inputs in folder
                            Hashtable port = (Hashtable)hash[i];
                            port.Add("InputID", i);
                            int typeNo = int.Parse(port["Type"].ToString());

                            if (!(typeNo == 1 || typeNo == 4))
                            {
                                if (port["FolderName"].ToString() != "")
                                {
                                    if (!_dvmNode.Nodes.ContainsKey(port["FolderName"].ToString()))
                                    {
                                        _dvmNode.Nodes.Add(port["FolderName"].ToString(), port["FolderName"].ToString(), IconImagelist.CLOSED_FOLDER, IconImagelist.CLOSED_FOLDER);
                                    }
                                    node = _dvmNode.Nodes[port["FolderName"].ToString()];
                                }
                                node.Nodes.Add("Name", port["Name"].ToString(), 0, 0);
                                node.LastNode.Tag = port; 
                                node = _dvmNode;
                            }
                        }
                    }
                }
                for (int i = 0; i < _dvmNode.GetNodeCount(false); i++)
                { // Go through all folders and inputs without folders
                    int syncCnt = 0; // Count TS Sync Loss
                    int nmCnt = 0; // Count No Monitoring
                    int disCnt = 0; // Count disconnect analyzer
                    int redCnt = 0; // Count the red LED (Current Alarm)
                    int yelCnt = 0; // Count the yellow LED (alarm in past)
                    int greCnt = 0; // No alarm
                    int warHisCnt = 0; // Count the Warnings in Past
                    int warCurCnt = 0; // Count the current Warnings

                    node = _dvmNode.Nodes[i];

                    if (node.GetNodeCount(false) > 0)
                    { // Folder
                        foreach (TreeNode input_port in node.Nodes)
                        { // Inputs
                            Hashtable port = (Hashtable)input_port.Tag;
                            int typeNo = int.Parse(port["Type"].ToString());

                            if (!(typeNo == 1 || typeNo == 4))
                            { // if it's no folder
                                int imgIndex = 0;
                                Int32 bitfield = Int32.Parse((port["AnalyzerPortBitfield"]).ToString());

                                if ((bitfield & 4) == 4 && ((bitfield & 2) != 2))
                                { //Monitoring stopped
                                    imgIndex = IconImagelist.NO_MONITORING;
                                    nmCnt++;
                                    dvmNode_nmCnt++;
                                }
                                else
                                { //Monitoring started
                                    if (bitfield != 0 && (bitfield & 2) != 2)
                                    { //Analyzer connected
                                        if ((bitfield & 128) == 128)
                                        { // Sync loss
                                            dvmNode_syncCnt++;
                                            syncCnt++;
                                            if ((bitfield & 16) == 16)
                                            { // Current Warning
                                                imgIndex = IconImagelist.SYNC_LOSS_CURRENT_WARNING;
                                                warCurCnt++;
                                            }
                                            else if ((bitfield & 8) == 8)
                                            { // Warning History
                                                imgIndex = IconImagelist.SYNC_LOSS_HISTORY_WARNING;
                                                warHisCnt++;
                                            }
                                            else
                                            { // no Warning
                                                imgIndex = IconImagelist.SYNC_LOSS;
                                            }
                                        }
                                        else if ((bitfield & 64) == 64 && (bitfield & 32) != 32)
                                        { // Alarm History
                                            yelCnt++;
                                            dvmNode_yelCnt++;
                                            if ((bitfield & 16) == 16)
                                            { // Current Warning
                                                imgIndex = IconImagelist.HISTORY_ALARM_CURRENT_WARNING;
                                                warCurCnt++;
                                            }
                                            else if ((bitfield & 8) == 8)
                                            { // Warning History
                                                imgIndex = IconImagelist.HISTORY_ALARM_HISTORY_WARNING;
                                                warHisCnt++;
                                            }
                                            else
                                            { // no Warning
                                                imgIndex = IconImagelist.HISTORY_ALARM;
                                            }
                                        }
                                        else if ((bitfield & 32) == 32 && ((bitfield & 64) == 64))
                                        { // Current Alarm
                                            redCnt++;
                                            dvmNode_redCnt++;
                                            if ((bitfield & 16) == 16)
                                            { // Current Warning
                                                imgIndex = IconImagelist.CURRENT_ALARM_CURRENT_WARNING;
                                                warCurCnt++;
                                            }
                                            else if ((bitfield & 8) == 8)
                                            { // Warning History
                                                imgIndex = IconImagelist.CURRENT_ALARM_HISTORY_WARNING;
                                                warHisCnt++;
                                            }
                                            else
                                            { // no Warning
                                                imgIndex = IconImagelist.CURRENT_ALARM;
                                            }
                                        }
                                        else if ((bitfield & 32) == 32)
                                        { // No Alarm
                                            greCnt++;
                                            dvmNode_greCnt++;
                                            if ((bitfield & 16) == 16)
                                            { // Current Warning
                                                imgIndex = IconImagelist.NO_ALARM_CURRENT_WARNING;
                                                warCurCnt++;
                                            }
                                            else if ((bitfield & 8) == 8)
                                            { // Warning History
                                                imgIndex = IconImagelist.NO_ALARM_HISTORY_WARNING;
                                                warHisCnt++;
                                            }
                                            else
                                            { // no Warning
                                                imgIndex = IconImagelist.NO_ALARM;
                                            }
                                        }
                                    }
                                    else
                                    { //Analyzer disconnected
                                        imgIndex = IconImagelist.ANALYZER_DISCONNECT;
                                        disCnt++;
                                        dvmNode_disCnt++;
                                    }
                                }
                                input_port.ImageIndex = input_port.SelectedImageIndex = imgIndex;
                                input_port.Tag = this._ip + "|" + this._rComm + '|' + this._wComm + "|" + port["InputID"].ToString() + "|" + port["AnalyzerId"].ToString();
                            }
                        }

                        // Closed Folder Icon
                        if (syncCnt > 0 && disCnt > 0 && warCurCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_SYNC_LOSS_DISCONNECTED_CURRENT_WARNING_NO_MONITORING;
                        }
                        else if (syncCnt > 0 && disCnt > 0 && warHisCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_SYNC_LOSS_DISCONNECTED_HISTORY_WARNING_NO_MONITORING;
                        }
                        else if (redCnt > 0 && disCnt > 0 && warCurCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_CURRENT_ALARM_DISCONNECTED_CURRENT_WARNING_NO_MONITORING;
                        }
                        else if (redCnt > 0 && disCnt > 0 && warHisCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_CURRENT_ALARM_DISCONNECTED_HISTORY_WARNING_NO_MONITORING;
                        }
                        else if (yelCnt > 0 && disCnt > 0 && warCurCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_HISTORY_ALARM_DISCONNECTED_CURRENT_WARNING_NO_MONITORING;
                        }
                        else if (yelCnt > 0 && disCnt > 0 && warHisCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_HISTORY_ALARM_DISCONNECTED_HISTORY_WARNING_NO_MONITORING;
                        }
                        else if (greCnt > 0 && disCnt > 0 && warCurCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_NO_ALARM_DISCONNECTED_CURRENT_WARNING_NO_MONITORING;
                        }
                        else if (greCnt > 0 && disCnt > 0 && warHisCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_NO_ALARM_DISCONNECTED_HISTORY_WARNING_NO_MONITORING;
                        }
                        else if (syncCnt > 0 && disCnt > 0 && warCurCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_SYNC_LOSS_CURRENT_WARNING_DISCONNECT;
                        }
                        else if (syncCnt > 0 && disCnt > 0 && warHisCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_SYNC_LOSS_HISTORY_WARNING_DISCONNECT;
                        }
                        else if (redCnt > 0 && disCnt > 0 && warCurCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_CURRENT_ALARM_CURRENT_WARNING_DISCONNECT;
                        }
                        else if (redCnt > 0 && disCnt > 0 && warHisCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_CURRENT_ALARM_HISTORY_WARNING_DISCONNECT;
                        }
                        else if (yelCnt > 0 && disCnt > 0 && warCurCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_HISTORY_ALARM_CURRENT_WARNING_DISCONNECT;
                        }
                        else if (yelCnt > 0 && disCnt > 0 && warHisCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_HISTORY_ALARM_HISTORY_WARNING_DISCONNECT;
                        }
                        else if (greCnt > 0 && disCnt > 0 && warCurCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_NO_ALARM_CURRENT_WARNING_DISCONNECT;
                        }
                        else if (greCnt > 0 && disCnt > 0 && warHisCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_NO_ALARM_HISTORY_WARNING_DISCONNECT;
                        }
                        else if (syncCnt > 0 && nmCnt > 0 && disCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_SYNC_LOSS_NO_MONITORING_DISCONNECTED;
                        }
                        else if (redCnt > 0 && nmCnt > 0 && disCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_CURRENT_ALARM_NO_MONITORING_DISCONNECTED;
                        }
                        else if (yelCnt > 0 && nmCnt > 0 && disCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_HISTORY_ALARM_NO_MONITORING_DISCONNECTED;
                        }
                        else if (greCnt > 0 && nmCnt > 0 && disCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_NO_ALARM_NO_MONITORING_DISCONNECTED;
                        }
                        else if (syncCnt > 0 && warCurCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_SYNC_LOSS_CURRENT_WARNING_NO_MONITORING;
                        }
                        else if (syncCnt > 0 && warHisCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_SYNC_LOSS_HISTORY_WARNING_NO_MONITORING;
                        }
                        else if (redCnt > 0 && warCurCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_CURRENT_ALARM_CURRENT_WARNING_NO_MONITORING;
                        }
                        else if (redCnt > 0 && warHisCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_CURRENT_ALARM_HISTORY_WARNING_NO_MONITORING;
                        }
                        else if (yelCnt > 0 && warCurCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_HISTORY_ALARM_CURRENT_WARNING_NO_MONITORING;
                        }
                        else if (yelCnt > 0 && warHisCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_HISTORY_ALARM_HISTORY_WARNING_NO_MONITORING;
                        }
                        else if (greCnt > 0 && warCurCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_NO_ALARM_CURRENT_WARNING_NO_MONITORING;
                        }
                        else if (greCnt > 0 && warHisCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_NO_ALARM_HISTORY_WARNING_NO_MONITORING;
                        }
                        else if (syncCnt > 0 && warCurCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_SYNC_LOSS_CURRENT_WARNING;
                        }
                        else if (syncCnt > 0 && warHisCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_SYNC_LOSS_HISTORY_WARNING;
                        }
                        else if (greCnt > 0 && warCurCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_NO_ALARM_CURRENT_WARNING_NO_MONITORING;
                        }
                        else if (greCnt > 0 && warHisCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_NO_ALARM_HISTORY_WARNING_NO_MONITORING;
                        }
                        else if (syncCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_SYNC_LOSS_NO_MONITORING;
                        }
                        else if (syncCnt > 0 && disCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_SYNC_LOSS_DISCONNECTED;
                        }
                        else if (redCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_CURRENT_ALARM_NO_MONITORING;
                        }
                        else if (redCnt > 0 && warCurCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_CURRENT_ALARM_CURRENT_WARNING;
                        }
                        else if (redCnt > 0 && warHisCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_CURRENT_ALARM_HISTORY_WARNING;
                        }
                        else if (yelCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_HISTORY_ALARM_NO_MONITORING;
                        }
                        else if (yelCnt > 0 && warCurCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_HISTORY_ALARM_CURRENT_WARNING;
                        }
                        else if (yelCnt > 0 && warHisCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_HISTORY_ALARM_HISTORY_WARNING;
                        }
                        else if (greCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_NO_ALARM_NO_MONITORING;
                        }
                        else if (greCnt > 0 && warCurCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_NO_ALARM_CURRENT_WARNING;
                        }
                        else if (greCnt > 0 && warHisCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_NO_ALARM_HISTORY_WARNING;
                        }
                        else if (disCnt > 0 && nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_DISCONNECTED_NO_MONITORING;
                        }
                        else if (disCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_DISCONNECT;
                        }
                        else if (syncCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_SYNC_LOSS;
                        }
                        else if (redCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_CURRENT_ALARM;
                        }
                        else if (yelCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_HISTORY_ALARM;
                        }
                        else if (nmCnt > 0)
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_NO_MONITORING;
                        }
                        else
                        {
                            node.ImageIndex = node.SelectedImageIndex = IconImagelist.CLOSED_FOLDER_NO_ALARM_NO_WARNING;
                        }
                        node.Tag = node.ImageIndex;
                    }
                    else
                    { // Inputs without folders
                        Hashtable port = (Hashtable)node.Tag;
                        int typeNo = int.Parse(port["Type"].ToString());

                        if (!(typeNo == 1 || typeNo == 4))
                        {
                            int imgIndex = 0;
                            Int32 bitfield = Int32.Parse((port["AnalyzerPortBitfield"]).ToString());

                            if ((bitfield & 4) == 4 && ((bitfield & 2) != 2))
                            { //Monitoring stopped
                                imgIndex = IconImagelist.NO_MONITORING;
                                nmCnt++;
                                dvmNode_nmCnt++;
                            }
                            else
                            { //Monitoring started
                                if (bitfield != 0 && (bitfield & 2) != 2)
                                { //Analyzer connected
                                    if ((bitfield & 128) == 128)
                                    { // Sync loss
                                        dvmNode_syncCnt++;
                                        if ((bitfield & 16) == 16)
                                        { // Current Warning
                                            imgIndex = IconImagelist.SYNC_LOSS_CURRENT_WARNING;
                                        }
                                        else if ((bitfield & 8) == 8)
                                        { // Warning History
                                            imgIndex = IconImagelist.SYNC_LOSS_HISTORY_WARNING;
                                        }
                                        else
                                        { // no Warning
                                            imgIndex = IconImagelist.SYNC_LOSS;
                                        }
                                    }
                                    else if ((bitfield & 64) == 64 && (bitfield & 32) != 32)
                                    { // Alarm History
                                        dvmNode_yelCnt++;
                                        if ((bitfield & 16) == 16)
                                        { // Current Warning
                                            imgIndex = IconImagelist.HISTORY_ALARM_CURRENT_WARNING;
                                        }
                                        else if ((bitfield & 8) == 8)
                                        { // Warning History
                                            imgIndex = IconImagelist.HISTORY_ALARM_HISTORY_WARNING;
                                        }
                                        else
                                        { // no Warning
                                            imgIndex = IconImagelist.HISTORY_ALARM;
                                        }
                                    }
                                    else if ((bitfield & 32) == 32 && ((bitfield & 64) == 64))
                                    { // Current Alarm
                                        dvmNode_redCnt++;
                                        if ((bitfield & 16) == 16)
                                        { // Current Warning
                                            imgIndex = IconImagelist.CURRENT_ALARM_CURRENT_WARNING;
                                        }
                                        else if ((bitfield & 8) == 8)
                                        { // Warning History
                                            imgIndex = IconImagelist.CURRENT_ALARM_HISTORY_WARNING;
                                        }
                                        else
                                        { // no Warning
                                            imgIndex = IconImagelist.CURRENT_ALARM;
                                        }
                                    }
                                    else if ((bitfield & 32) == 32)
                                    { // No Alarm
                                        dvmNode_greCnt++;
                                        if ((bitfield & 16) == 16)
                                        { // Current Warning
                                            imgIndex = IconImagelist.NO_ALARM_CURRENT_WARNING;
                                        }
                                        else if ((bitfield & 8) == 8)
                                        { // Warning History
                                            imgIndex = IconImagelist.NO_ALARM_HISTORY_WARNING;
                                        }
                                        else
                                        { // no Warning
                                            imgIndex = IconImagelist.NO_ALARM;
                                        }
                                    }
                                }
                                else
                                { //Analyzer disconnected
                                    imgIndex = IconImagelist.ANALYZER_DISCONNECT;
                                }
                            }
                            node.ImageIndex = node.SelectedImageIndex = imgIndex;
                            node.Tag = this._ip + "|" + this._rComm + '|' + this._wComm + "|" + port["InputID"].ToString() + "|" + port["AnalyzerId"].ToString();
                        }
                    }
                }

                if (dvmNode_syncCnt > 0)
                {
                    _dvmNode.ImageIndex = _dvmNode.SelectedImageIndex = IconImagelist.MAP_TS_SYNC;
                }
                else if (dvmNode_redCnt > 0)
                {
                    _dvmNode.ImageIndex = _dvmNode.SelectedImageIndex = IconImagelist.MAP_CURRENT_ALARM;
                }
                else if (dvmNode_yelCnt > 0)
                {
                    _dvmNode.ImageIndex = _dvmNode.SelectedImageIndex = IconImagelist.MAP_HISTORY_ALARM;
                }
                else if(dvmNode_greCnt > 0)
                {
                    _dvmNode.ImageIndex = _dvmNode.SelectedImageIndex = IconImagelist.MAP_NO_ALARM;
                }
                else if (dvmNode_nmCnt > 0)
                {
                    _dvmNode.ImageIndex = _dvmNode.SelectedImageIndex = IconImagelist.MAP_NO_MONITORING;
                }
                else if (dvmNode_disCnt > 0)
                {
                    _dvmNode.ImageIndex = _dvmNode.SelectedImageIndex = IconImagelist.MAP_CONNECTIVITY_LOSS;
                }
            }
            else
            { // Connectivity loss
                _siteName = "--" + _ip + "--";
                _dvmNode.Text = _siteName;
                _led.ImageIndex = _dvmNode.ImageIndex = _dvmNode.SelectedImageIndex = IconImagelist.MAP_CONNECTIVITY_LOSS;
            } 
            _ready = true;             
        }
    }
}
