/*Class name: DVMAppFuncs
 *Namespace: DVM_Management_Center.SNMP
 *Author: Harald Gsoedl
 *Last Change: Benno Rott
 *Changed: 21/02/2007
 *Use: Provides routines to read application relevant information from the dvm.
 */

using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace DVM_Management_Center.SNMP
{
    class DVMAppFuncs:DVMBasics
    {
        public DVMAppFuncs(string ip, string readCommunity, string writeCommunity, string mibPath):base(ip,readCommunity,writeCommunity,mibPath)
        {
        }

        private Hashtable _inpInfos;
        public Hashtable InpInfos
        {
            get { return _inpInfos; }
            set { _inpInfos = value; }
        }

        /// <summary>
        /// Methode to read all informations from the DVM
        /// </summary>
        public void SetAllInputInfos()
        {
            try
            {
                int analCnt = int.Parse(this.GetAnalyzerCount());

                Hashtable analyzers = new Hashtable();
                analyzers.Add(0, this.GetSiteName());
                for (int i = 1; i <= analCnt; i++)
                {
                    string macAnal = this.GetAnalyzerMACs(i);
                    string macAnal_oid = this.GetOidFromToken(macAnal);

                    Hashtable hash = new Hashtable();
                    for (int j = 4; j >= 1; j--)
                    {
                        Hashtable temp = this.GetAllInformation(macAnal_oid, j);
                        Hashtable portInfos = new Hashtable();
                        portInfos.Add("Name", ((Hashtable)((Hashtable)temp)[1])["value"]);
                        portInfos.Add("Type", ((Hashtable)((Hashtable)temp)[2])["value"]);
                        portInfos.Add("Config", ((Hashtable)((Hashtable)temp)[3])["value"]);
                        portInfos.Add("LED", ((Hashtable)((Hashtable)temp)[4])["value"]);
                        
                        portInfos.Add("FolderName", ((Hashtable)((Hashtable)temp)[5])["value"]);
                        if (portInfos["FolderName"].ToString() == "MAC Address")
                            portInfos["FolderName"] = macAnal;
                        
                        portInfos.Add("AnalyzerId", i);

                        Byte[] ba = Encoding.UTF8.GetBytes(((Hashtable)((Hashtable)temp)[6])["value"].ToString());
                        portInfos.Add("AnalyzerPortBitfield", ba[ba.Length - 2].ToString());
                        
                        hash.Add(j, portInfos);
                    }
                    analyzers.Add(i, hash);
                 }
                 
                _inpInfos = analyzers;
            }
            catch
            {
                _inpInfos = null;
            }
        }

        public void ClearLogs(int portId, int analyzerId)
        {
            string macAnal = this.GetAnalyzerMACs(analyzerId);
            macAnal = this.GetOidFromToken(macAnal);
            SetClearLogs(macAnal, portId);
        }

    }
}
